from file_handlers.lumps.structure_parser import Struct
from file_handlers.lumps.header import HeaderParser, HeaderPacker
from file_handlers.lumps.l35 import Lump35Parser, Lump35Packer
from pathlib import Path
import tempfile

class DeleteL27():
    def __init__(self, fp):
        self.__headerparser = HeaderParser(fp)
        self.__headerpacker = HeaderPacker(fp)

        self.__fp = fp
        self.__lumps_to_remove = [27, 51, 52]

    def delete_l27(self):
        self.__fp.seek(0)
        hdata = self.__headerparser.parse()

        offsets_list = []
        lnum = 0
        for lump in hdata["lump_t"]:
            offsets_list.append([lump["fileofs"], lump["filelen"], lnum])
            lnum += 1
        offsets_list = sorted(offsets_list)

        for lump in self.__lumps_to_remove:
            for x in range(63):
                if offsets_list[x][2] == lump and offsets_list[x][0]:
                    ofs = hdata["lump_t"][lump]["fileofs"]
                    llen = hdata["lump_t"][lump]["filelen"]
                    hdata["lump_t"][lump]["fileofs"] = 0
                    hdata["lump_t"][lump]["filelen"] = 0

                    for item in offsets_list[x+1:]:
                        if item[2] == 35:
                            print("%i: %i" %(item[2], hdata["lump_t"][item[2]]["fileofs"]))

                            self.__fp.seek(item[0])
                            l35parser = Lump35Parser(self.__fp)
                            self.__fp.seek(item[0])
                            l35packer = Lump35Packer(self.__fp)
                            
                            l35data = l35parser.parse_header()

                            for nitem in l35data["dgamelump_t"]:
                                if nitem["fileofs"]:
                                    nitem["fileofs"] -= llen
                            l35packer.pack_header(l35data)

                        if hdata["lump_t"][item[2]]["fileofs"]:
                            if hdata["lump_t"][item[2]]["fileofs"] >= llen:
                                hdata["lump_t"][item[2]]["fileofs"] -= llen
                            else:
                                hdata["lump_t"][item[2]]["fileofs"] = 0
##                        print("%i: %i" %(item[2], hdata["lump_t"][item[2]]["fileofs"]))
                    break

            self.__fp.seek(0)

            with tempfile.TemporaryFile() as tf:
                tf.write(self.__fp.read(ofs))
                self.__fp.seek(llen, 1)
                tf.write(self.__fp.read())

                tf.seek(0)
                self.__fp.seek(0)
                self.__fp.truncate()
                self.__fp.write(tf.read())
                
                self.__fp.seek(0)
                self.__headerpacker.pack(hdata)
            

            
