from .structure_parser import Struct
from .lump_class_base import LumpBaseParser, LumpBasePacker

DGL = Struct("dgamelump",
                (("id", "RAW/4"), ##raw, unlike most IDs, so we can create the dummy gamelump in repacking
                 ("flags", "SHORTUL"),
                 ("version", "SHORTUL"),
                 ("fileofs", "INTUL"),
                 ("filelen", "INTUL")))
DGLH = Struct("dgamelumpheader",
                (("lumpCount", "INTUL"),
                 ("dgamelump_t", "dgamelump/lumpCount")),
                DGL)
modelentry = Struct("modelentry",
                (("name", "RAW/128"),))
modeldata = Struct("modeldata",
                (("dictEntries", "INTUL"),
                 ("modelNames", "modelentry/dictEntries")),
                modelentry)
spl_t = Struct("spl_t",
                (("misc_data", "RAW/24"),
                 ("PropType", "SHORTUL"), ##index into model name dict
                 ("misc_data2", "RAW/6"), ##firstleaf, leafcount, solid, flags
                 ("Skin", "INTUL"),
                 ("misc_data3", "RAW/36")))
##misc_data3 is made up of fademin/maxdist (float), lightingorigin (vec3),
##fadescale (float), min/maxcpu/gpulevel(chars), diffusemodulation (color32),
##and flagsEx (int)


##24 + color32 + lightmapresx/y
prps = Struct("prop_static_data", ##lump version 10
                (("leafEntries", "INTUL"),
                 ("leaf", "SHORTUL/leafEntries"), ##end of leaf dict
                 ("nStaticPropLumps", "INTUL"),
                 ("StaticPropLump_t", "spl_t/nStaticPropLumps")),
                spl_t)   

class Lump35Packer(LumpBasePacker):
    def __init__(self, fp):
        super().__init__(fp)

    def pack_header(self, nheader, delta_ngamelumps = 0):
        self._init_fp()
        if delta_ngamelumps > nheader["lumpCount"]:
            raise ValueError("Invalid delta_ngamelumps value %i" %delta_ngamelumps)
        DGLH.pack(self._fp, nheader, 4+(nheader["lumpCount"] - delta_ngamelumps)*16)
        ##deltangamelumps is used for when the size of the header would change;
        ##let deltangamelumps be equal to the number of game lumps more that the new header has
        ##(this is negative for fewer game lumps, and positive for more game lumps)

class Lump35Parser(LumpBaseParser):
    def __init__(self, fp, length=-1):
        super().__init__(fp, length)

    def parse_header(self):
        self._init_fp()
        data = DGLH.parse(self._fp)
        return data

    def parse_skins(self):
        print("parsing skins..")
        self._init_fp()
        dglh = DGLH.parse(self._fp)
        sk_output = {}
        for x in dglh["dgamelump_t"]:
            if x["id"] == b"prps":
                if x["version"] not in [10, ]:
                    raise ValueError("Static prop gamelump version is not in the supported list. Actual version is %i." %x["version"])
                    ##I doubt this is necessary but leaving it just in case
                self._fp.seek(x["fileofs"], 0)
                data = modeldata.parse(self._fp)["modelNames"]
                prps_data = prps.parse(self._fp)
                for item in prps_data["StaticPropLump_t"]:
                    mdlname = data[item["PropType"]]["name"].replace(b"\x00", b"")
                    if mdlname not in sk_output:
                        sk_output[mdlname] = set()
                    sk_output[mdlname].add(item["Skin"])
                    ##check this code. IDs seem to be a bit dodgy, not existing in the actual models at times
                break
            
        ##return a dictionary mapping the model name to a set of skin used
        print("done")
        return sk_output

    def parse(self, raw = False):
        self._init_fp()
        dglh = DGLH.parse(self._fp)
        model_array = {}
        if not raw:
            for x in dglh["dgamelump_t"]: #for each sublump
                self._fp.seek(x["fileofs"], 0)
                if x["id"] in (b"prps", b"prpd"):
                    data = modeldata.parse(self._fp)["modelNames"]
                    model_array[x["id"]] = map((lambda data: data["name"].replace(b"\x00", b"")), data)
                    ##set not needed as each entry is unique

        dglh["data_array"] = model_array
        
        return dglh
